/* Jcode - japanese character code conversion
 * Copyright (C) 2000 Specific Phone Systems / timecop
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#include "filter.h"

/* Jcode::hantozen(string,[options,[code]])
 * string is the string to convert
 * options list from the README file (optional)
 * optional code parameter identifies string encoding,
 * and if omitted, it is attempted to be automatically guessed
 */

XS(XS_Jcode_hantozen)
{
    dXSARGS;
    char *in, *option, *code, *ret;
    int opt = 0, n = 0, fenc = 0, i = 0;

    switch (items) {
    case 0:
	croak("hantozen called with too few arguments");
	break;
    case 1:
	in = (char *) SvPV(ST(0), PL_na);
	option = "KV";
	break;
    case 2:
	in = (char *) SvPV(ST(0), PL_na);
	option = (char *) SvPV(ST(1), PL_na);
	break;
    case 3:
	in = (char *) SvPV(ST(0), PL_na);
	option = (char *) SvPV(ST(1), PL_na);
	code = (char *) SvPV(ST(2), PL_na);
	n = string2enc(code);
	fenc = 1;
	break;
    }
    while (i < strlen(option)) {
	switch (option[i++]) {
	case 'A':
	    opt |= 0x1;
	    break;
	case 'a':
	    opt |= 0x10;
	    break;
	case 'R':
	    opt |= 0x2;
	    break;
	case 'r':
	    opt |= 0x20;
	    break;
	case 'N':
	    opt |= 0x4;
	    break;
	case 'n':
	    opt |= 0x40;
	    break;
	case 'K':
	    opt |= 0x100;
	    break;
	case 'k':
	    opt |= 0x1000;
	    break;
	case 'H':
	    opt |= 0x200;
	    break;
	case 'h':
	    opt |= 0x2000;
	    break;
	case 'V':
	    opt |= 0x800;
	    break;
	case 'C':
	    opt |= 0x10000;
	    break;
	case 'c':
	    opt |= 0x20000;
	    break;
	}
    }
    if (!fenc)
	n = guess2enc(in);
    ret = mbfl_ja_jp_hantozen(in, opt, n);
    ST(0) = sv_newmortal();
    sv_setpv((SV *) ST(0), ret);
    XSRETURN(1);
}

/* Jcode::mblength(string,[code])
 * string is the string to be measured
 * optional code parameter identifies string encoding,
 * and if omitted, it is attempted to be automatically guessed
 */

XS(XS_Jcode_mblength)
{
    dXSARGS;
    char *in, *code;
    int n, len;

    switch (items) {
    case 0:
	croak("mblength called with too few arguments");
	break;
    case 1:
	in = (char *) SvPV(ST(0), PL_na);
	n = guess2enc(in);
	break;
    case 2:
	in = (char *) SvPV(ST(0), PL_na);
	code = (char *) SvPV(ST(1), PL_na);
	n = string2enc(code);
	break;
    }
    len = mbfl_strlen(in, n);
    ST(0) = sv_newmortal();
    sv_setiv(ST(0), (IV) len);
    XSRETURN(1);
}

/* Jcode::discover(string)
 * string is the string to guess the encoding of
 */

XS(XS_Jcode_discover)
{
    dXSARGS;
    char *lang, *in;
    mbfl_encoding_ptr penc;

    switch (items) {
    case 0:
	croak("discover called with too few arguments");
	break;
    case 1:
	in = (char *) SvPV(ST(0), PL_na);
	break;
    }
    penc = mbfl_no2encoding(mbfl_identify_encoding(in));
    if (penc != NULL && penc->no_encoding != mbfl_no_encoding_auto)
	lang = (char *) penc->name;
    else
	lang = "unknown";

    ST(0) = sv_newmortal();
    sv_setpv((SV *) ST(0), lang);
    XSRETURN(1);
}

/* Jcode::convert(string,code,[oldcode])
 * string is the string to convert
 * code is the code to convert the encoding into
 * optional oldcode parameter identifies string encoding,
 * and if omitted, it is attempted to be automatically guessed
 */

XS(XS_Jcode_convert)
{
    dXSARGS;
    int newcode, oldcode;
    char *in, *ret, *code;

    switch (items) {
    case 0:
    case 1:
	croak("convert called with too few arguments");
	break;
    case 2:
	in = (char *) SvPV(ST(0), PL_na);
	code = (char *) SvPV(ST(1), PL_na);
	newcode=string2enc(code);
	oldcode=guess2enc(in);
	break;
    case 3:
	in = (char *) SvPV(ST(0), PL_na);
	code = (char *) SvPV(ST(1), PL_na);
	newcode=string2enc(code);
	code = (char *) SvPV(ST(2), PL_na);
	oldcode=string2enc(code);
	break;
    }
    ret = mbfl_encoding_convert(oldcode, newcode, in);
    ST(0) = sv_newmortal();
    sv_setpv((SV *) ST(0), ret);
    XSRETURN(1);
}

/* Jcode::mbsendmail(from,to,[subject],message)
 * from is a valid "From" email address
 * to is a valid "To" email address
 * optional subject parameter will set the Subject: line if provided
 * message is the body of the message to be sent.
 * In all cases, encoding is automatically guessed.  Output is encoded into
 * ISO-2022-JP and/or US-ASCII depending on contents of the body
 */

XS(XS_Jcode_mbsendmail)
{
    dXSARGS;
    char *from;
    char *to;
    char *subject;
    char *message;
    char temp[65000];
    char *temp1;
    char *enc_text = NULL;
    int code,len;

    switch (items) {
    default:
	croak("mbsendmail called with too few arguments");
	break;
    case 3:
        from= (char *) SvPV(ST(0), PL_na);
        to = (char *) SvPV(ST(1), PL_na);
        message = (char *) SvPV(ST(2), PL_na);
	subject = NULL;
        break;
    case 4:
        from= (char *) SvPV(ST(0), PL_na);
        to = (char *) SvPV(ST(1), PL_na);
        subject = (char *) SvPV(ST(2), PL_na);
        message = (char *) SvPV(ST(3), PL_na);
        break;
    }
    
    if (strlen(message) > 61440)
     croak("message size exceeds 60 kb");

    /* from/to */
    sprintf(temp, "From: %s\n", from);
    temp1=malloc(strlen(to)+250);
    sprintf(temp1, "To: %s\n", to);
    strcat(temp,temp1);
    free(temp1);

    if (subject) {
	/* subject */
	code = mbfl_identify_encoding(subject);
	enc_text = mbfl_mime_header_encode(
		code,
		mbfl_no_encoding_ja_jp_jis,
		mbfl_no_encoding_base64,
		subject,
		"\n");
	temp1=malloc(strlen(enc_text)+250);
	sprintf(temp1,"Subject: %s\n", enc_text);
	free(enc_text);
	strcat(temp,temp1);
	free(temp1);
     }

	/* body */
	strcat(temp, "Mime-Version: 1.0\n");
	code = mbfl_identify_encoding(message);
	temp1=malloc(256*sizeof(char));
	if(code == mbfl_no_encoding_en_us_ascii) {
		sprintf(temp1, "Content-Type: text/plain; charset=US-ASCII\nContent-Transfer-Encoding: 7bit\n\n");
		strcat(temp,temp1);
		free(temp1);
		strcat(temp,message);
	} else {
		temp1=malloc(256*sizeof(char));
		sprintf(temp1, "Content-Type: text/plain; charset=iso-2022-jp\nContent-Transfer-Encoding: 7bit\n\n");
		strcat(temp,temp1);
		free(temp1);
		/* simple jis conversion */
		enc_text = mbfl_encoding_convert(
			code,
			mbfl_no_encoding_ja_jp_jis,
			message);
		strcat(temp,enc_text);
		free(enc_text);
	}

	strcat(temp,"\n");

    ST(0) = sv_newmortal();
    sv_setpv((SV *) ST(0), temp);
    XSRETURN(1);
}

XS(boot_Jcode)
{
    dXSARGS;
    char *file = __FILE__;

    newXS("Jcode::hantozen", XS_Jcode_hantozen, file);
    newXS("Jcode::mblength", XS_Jcode_mblength, file);
    newXS("Jcode::discover", XS_Jcode_discover, file);
    newXS("Jcode::convert", XS_Jcode_convert, file);
    newXS("Jcode::mbsendmail", XS_Jcode_mbsendmail, file);
    ST(0) = &PL_sv_yes;
    XSRETURN(1);
}

/* helper functions */
int string2enc(code)
char *code;
{
    mbfl_encoding_ptr penc;

    penc = mbfl_name2encoding(code);
    if (penc != NULL)
	return penc->no_encoding;
    else
	croak("unknown encoding \"%s\"", code);
    return -1;			/* not reached */
}

int guess2enc(in)
char *in;
{
    mbfl_encoding_ptr penc;
    penc = mbfl_no2encoding(mbfl_identify_encoding(in));
    if (penc != NULL && penc->no_encoding != mbfl_no_encoding_auto)
	return penc->no_encoding;
    else
	croak("cannot guess encoding");
    return -1;			/* not reached */
}
